@@ -82,13 +82,23 @@ class Box:
 
     # Default, overriden on some subclasses
     def all_children(self):
-        return ()
+        return self.children
+
+    def descendants(self, placeholders=False):
+        """A flat generator for a box, its children and descendants."""
+        yield self
+        for child in self.children:
+            if placeholders or isinstance(child, Box):
+                yield from child.descendants(placeholders)
+            else:
+                yield child
 
     def __init__(self, element_tag, style, element):
         self.element_tag = element_tag
         self.element = element
         self.style = style
         self.remove_decoration_sides = set()
+        self.children = []
 
     def __repr__(self):
         return f'<{type(self).__name__} {self.element_tag}>'
@@ -332,9 +342,6 @@ class ParentBox(Box):
         super().__init__(element_tag, style, element)
         self.children = tuple(children)
 
-    def all_children(self):
-        return self.children
-
     def _reset_spacing(self, side):
         """Set to 0 the margin, padding and border of ``side``."""
         self.remove_decoration_sides.add(side)
@@ -353,7 +360,7 @@ class ParentBox(Box):
     def copy_with_children(self, new_children):
         """Create a new equivalent box with given ``new_children``."""
         new_box = self.copy()
-        new_box.children = list(new_children)
+        new_box.children = new_children
 
         # Clear and reset removed decorations as we don't want to keep the
         # previous data, for example when a box is split between two pages.
@@ -363,19 +370,9 @@ class ParentBox(Box):
 
     def deepcopy(self):
         result = self.copy()
-        result.children = tuple(child.deepcopy() for child in self.children)
+        result.children = list(child.deepcopy() for child in self.children)
         return result
 
-    def descendants(self):
-        """A flat generator for a box, its children and descendants."""
-        yield self
-        for child in self.children:
-            if isinstance(child, ParentBox):
-                for grand_child in child.descendants():
-                    yield grand_child
-            else:
-                yield child
-
     def get_wrapped_table(self):
         """Get the table wrapped by the box."""
         assert self.is_table_wrapper
