@@ -23,6 +23,12 @@ class OrientedBox:
     def outer(self):
         return self.sugar + self.inner
 
+    @outer.setter
+    def outer(self, new_outer_width):
+        self.inner = min(
+            max(self.min_content_size, new_outer_width - self.sugar),
+            self.max_content_size)
+
     @property
     def outer_min_content_size(self):
         return self.sugar + (
@@ -33,10 +39,6 @@ class OrientedBox:
         return self.sugar + (
             self.max_content_size if self.inner == 'auto' else self.inner)
 
-    def shrink_to_fit(self, available):
-        self.inner = min(
-            max(self.min_content_size, available), self.max_content_size)
-
 
 class VerticalBox(OrientedBox):
     def __init__(self, context, box):
@@ -102,9 +104,9 @@ class HorizontalBox(OrientedBox):
 
 
 def compute_fixed_dimension(context, box, outer, vertical, top_or_left):
-    """
-    Compute and set a margin box fixed dimension on ``box``, as described in:
-    http://dev.w3.org/csswg/css3-page/#margin-constraints
+    """Compute and set a margin box fixed dimension on ``box``.
+
+    Described in: https://drafts.csswg.org/css-page-3/#margin-constraints
 
     :param box:
         The margin box to work on
@@ -123,7 +125,7 @@ def compute_fixed_dimension(context, box, outer, vertical, top_or_left):
 
     # Rule 2
     total = box.padding_plus_border + sum(
-        value for value in [box.margin_a, box.margin_b, box.inner]
+        value for value in (box.margin_a, box.margin_b, box.inner)
         if value != 'auto')
     if total > outer:
         if box.margin_a == 'auto':
@@ -134,7 +136,7 @@ def compute_fixed_dimension(context, box, outer, vertical, top_or_left):
             # XXX this is not in the spec, but without it box.inner
             # would end up with a negative value.
             # Instead, this will trigger rule 3 below.
-            # http://lists.w3.org/Archives/Public/www-style/2012Jul/0006.html
+            # https://lists.w3.org/Archives/Public/www-style/2012Jul/0006.html
             box.inner = 0
     # Rule 3
     if 'auto' not in [box.margin_a, box.margin_b, box.inner]:
@@ -163,7 +165,7 @@ def compute_fixed_dimension(context, box, outer, vertical, top_or_left):
         box.inner = (outer - box.padding_plus_border -
                      box.margin_a - box.margin_b)
     # Rule 6
-    if box.margin_a == 'auto' and box.margin_b == 'auto':
+    if box.margin_a == box.margin_b == 'auto':
         box.margin_a = box.margin_b = (
             outer - box.padding_plus_border - box.inner) / 2
 
@@ -172,21 +174,22 @@ def compute_fixed_dimension(context, box, outer, vertical, top_or_left):
     box.restore_box_attributes()
 
 
-def compute_variable_dimension(context, side_boxes, vertical, outer_sum):
-    """
-    Compute and set a margin box fixed dimension on ``box``, as described in:
-    http://dev.w3.org/csswg/css3-page/#margin-dimension
+def compute_variable_dimension(context, side_boxes, vertical, available_size):
+    """Compute and set a margin box fixed dimension on ``box``
 
-    :param side_boxes: Three boxes on a same side (as opposed to a corner.)
+    Described in: https://drafts.csswg.org/css-page-3/#margin-dimension
+
+    :param side_boxes:
+        Three boxes on a same side (as opposed to a corner).
         A list of:
         - A @*-left or @*-top margin box
         - A @*-center or @*-middle margin box
         - A @*-right or @*-bottom margin box
     :param vertical:
-        True to set height, margin-top and margin-bottom; False for width,
-        margin-left and margin-right
-    :param outer_sum:
-        The target total outer dimension (max box width or height)
+        ``True`` to set height, margin-top and margin-bottom;
+        ``False`` for width, margin-left and margin-right.
+    :param available_size:
+        The distance between the page box’s left right border edges
 
     """
     box_class = VerticalBox if vertical else HorizontalBox
@@ -199,68 +202,121 @@ def compute_variable_dimension(context, side_boxes, vertical, outer_sum):
         if box.margin_b == 'auto':
             box.margin_b = 0
 
-    if box_b.box.is_generated:
-        if box_b.inner == 'auto':
-            ac_max_content_size = 2 * max(
-                box_a.outer_max_content_size, box_c.outer_max_content_size)
-            if outer_sum >= (
-                    box_b.outer_max_content_size + ac_max_content_size):
-                box_b.inner = box_b.max_content_size
-            else:
-                ac_min_content_size = 2 * max(
-                    box_a.outer_min_content_size,
-                    box_c.outer_min_content_size)
-                box_b.inner = box_b.min_content_size
-                available = outer_sum - box_b.outer - ac_min_content_size
-                if available > 0:
-                    weight_ac = ac_max_content_size - ac_min_content_size
-                    weight_b = (
-                        box_b.max_content_size - box_b.min_content_size)
-                    weight_sum = weight_ac + weight_b
-                    # By definition of max_content_size and min_content_size,
-                    # weights can not be negative. weight_sum == 0 implies that
-                    # max_content_size == min_content_size for each box, in
-                    # which case the sum can not be both <= and > outer_sum
-                    # Therefore, one of the last two 'if' statements would not
-                    # have lead us here.
-                    assert weight_sum > 0
-                    box_b.inner += available * weight_b / weight_sum
-        if box_a.inner == 'auto':
-            box_a.shrink_to_fit((outer_sum - box_b.outer) / 2 - box_a.sugar)
-        if box_c.inner == 'auto':
-            box_c.shrink_to_fit((outer_sum - box_b.outer) / 2 - box_c.sugar)
-    else:
+    if not box_b.box.is_generated:
         # Non-generated boxes get zero for every box-model property
         assert box_b.inner == 0
         if box_a.inner == box_c.inner == 'auto':
-            if outer_sum >= (
+            # A and C both have 'width: auto'
+            if available_size > (
                     box_a.outer_max_content_size +
                     box_c.outer_max_content_size):
-                box_a.inner = box_a.max_content_size
-                box_c.inner = box_c.max_content_size
+                # sum of the outer max-content widths
+                # is less than the available width
+                flex_space = (
+                    available_size -
+                    box_a.outer_max_content_size -
+                    box_c.outer_max_content_size)
+                flex_factor_a = box_a.outer_max_content_size
+                flex_factor_c = box_c.outer_max_content_size
+                flex_factor_sum = flex_factor_a + flex_factor_c
+                if flex_factor_sum == 0:
+                    flex_factor_sum = 1
+                box_a.outer = box_a.max_content_size + (
+                    flex_space * flex_factor_a / flex_factor_sum)
+                box_c.outer = box_c.max_content_size + (
+                    flex_space * flex_factor_c / flex_factor_sum)
+            elif available_size > (
+                    box_a.outer_min_content_size +
+                    box_c.outer_min_content_size):
+                # sum of the outer min-content widths
+                # is less than the available width
+                flex_space = (
+                    available_size -
+                    box_a.outer_min_content_size -
+                    box_c.outer_min_content_size)
+                flex_factor_a = (
+                    box_a.max_content_size - box_a.min_content_size)
+                flex_factor_c = (
+                    box_c.max_content_size - box_c.min_content_size)
+                flex_factor_sum = flex_factor_a + flex_factor_c
+                if flex_factor_sum == 0:
+                    flex_factor_sum = 1
+                box_a.outer = box_a.min_content_size + (
+                    flex_space * flex_factor_a / flex_factor_sum)
+                box_c.outer = box_c.min_content_size + (
+                    flex_space * flex_factor_c / flex_factor_sum)
             else:
-                box_a.inner = box_a.min_content_size
-                box_c.inner = box_c.min_content_size
-                available = outer_sum - box_a.outer - box_c.outer
-                if available > 0:
-                    weight_a = (
-                        box_a.max_content_size - box_a.min_content_size)
-                    weight_c = (
-                        box_c.max_content_size - box_c.min_content_size)
-                    weight_sum = weight_a + weight_c
-                    # By definition of max_content_size and min_content_size,
-                    # weights can not be negative. weight_sum == 0 implies that
-                    # max_content_size == min_content_size for each box, in
-                    # which case the sum can not be both <= and > outer_sum
-                    # Therefore, one of the last two 'if' statements would not
-                    # have lead us here.
-                    assert weight_sum > 0
-                    box_a.inner += available * weight_a / weight_sum
-                    box_c.inner += available * weight_c / weight_sum
-        elif box_a.inner == 'auto':
-            box_a.shrink_to_fit(outer_sum - box_c.outer - box_a.sugar)
-        elif box_c.inner == 'auto':
-            box_c.shrink_to_fit(outer_sum - box_a.outer - box_c.sugar)
+                # otherwise
+                flex_space = (
+                    available_size -
+                    box_a.outer_min_content_size -
+                    box_c.outer_min_content_size)
+                flex_factor_a = box_a.min_content_size
+                flex_factor_c = box_c.min_content_size
+                flex_factor_sum = flex_factor_a + flex_factor_c
+                if flex_factor_sum == 0:
+                    flex_factor_sum = 1
+                box_a.outer = box_a.min_content_size + (
+                    flex_space * flex_factor_a / flex_factor_sum)
+                box_c.outer = box_c.min_content_size + (
+                    flex_space * flex_factor_c / flex_factor_sum)
+        else:
+            # only one box has 'width: auto'
+            if box_a.inner == 'auto':
+                box_a.outer = available_size - box_c.outer
+            elif box_c.inner == 'auto':
+                box_c.outer = available_size - box_a.outer
+    else:
+        if box_b.inner == 'auto':
+            # resolve any auto width of the middle box (B)
+            ac_max_content_size = 2 * max(
+                box_a.outer_max_content_size, box_c.outer_max_content_size)
+            if available_size > (
+                    box_b.outer_max_content_size + ac_max_content_size):
+                flex_space = (
+                    available_size -
+                    box_b.outer_max_content_size -
+                    ac_max_content_size)
+                flex_factor_b = box_b.outer_max_content_size
+                flex_factor_ac = ac_max_content_size
+                flex_factor_sum = flex_factor_b + flex_factor_ac
+                if flex_factor_sum == 0:
+                    flex_factor_sum = 1
+                box_b.outer = box_b.max_content_size + (
+                    flex_space * flex_factor_b / flex_factor_sum)
+            else:
+                ac_min_content_size = 2 * max(
+                    box_a.outer_min_content_size, box_c.outer_min_content_size)
+                if available_size > (
+                        box_b.outer_min_content_size + ac_min_content_size):
+                    flex_space = (
+                        available_size -
+                        box_b.outer_min_content_size -
+                        ac_min_content_size)
+                    flex_factor_b = (
+                        box_b.max_content_size - box_b.min_content_size)
+                    flex_factor_ac = ac_max_content_size - ac_min_content_size
+                    flex_factor_sum = flex_factor_b + flex_factor_ac
+                    if flex_factor_sum == 0:
+                        flex_factor_sum = 1
+                    box_b.outer = box_b.min_content_size + (
+                        flex_space * flex_factor_b / flex_factor_sum)
+                else:
+                    flex_space = (
+                        available_size -
+                        box_b.outer_min_content_size -
+                        ac_min_content_size)
+                    flex_factor_b = box_b.min_content_size
+                    flex_factor_ac = ac_min_content_size
+                    flex_factor_sum = flex_factor_b + flex_factor_ac
+                    if flex_factor_sum == 0:
+                        flex_factor_sum = 1
+                    box_b.outer = box_b.min_content_size + (
+                        flex_space * flex_factor_b / flex_factor_sum)
+        if box_a.inner == 'auto':
+            box_a.outer = (available_size - box_b.outer) / 2
+        if box_c.inner == 'auto':
+            box_c.outer = (available_size - box_b.outer) / 2
 
     # And, we’re done!
     assert 'auto' not in [box.inner for box in side_boxes]
@@ -309,8 +365,10 @@ def make_margin_boxes(context, page, state):
 
         Return ``None`` if this margin box should not be generated.
 
-        :param at_keyword: which margin box to return, eg. '@top-left'
-        :param containing_block: as expected by :func:`resolve_percentages`.
+        :param at_keyword:
+            Which margin box to return, e.g. '@top-left'
+        :param containing_block:
+            As expected by :func:`resolve_percentages`.
 
         """
         style = context.style_for(page.page_type, at_keyword)
@@ -359,10 +417,10 @@ def make_margin_boxes(context, page, state):
     page_end_y = margin_top + max_box_height
 
     # Margin box dimensions, described in
-    # http://dev.w3.org/csswg/css3-page/#margin-box-dimensions
+    # https://drafts.csswg.org/css-page-3/#margin-box-dimensions
     generated_boxes = []
 
-    for prefix, vertical, containing_block, position_x, position_y in [
+    for prefix, vertical, containing_block, position_x, position_y in (
         ('top', False, (max_box_width, margin_top),
             margin_left, 0),
         ('bottom', False, (max_box_width, margin_bottom),
@@ -371,7 +429,7 @@ def make_margin_boxes(context, page, state):
             0, margin_top),
         ('right', True, (margin_right, max_box_height),
             page_end_x, margin_top),
-    ]:
+    ):
         if vertical:
             suffixes = ['top', 'middle', 'bottom']
             fixed_outer, variable_outer = containing_block
@@ -399,18 +457,18 @@ def make_margin_boxes(context, page, state):
                     variable_outer - box.margin_width())
             compute_fixed_dimension(
                 context, box, fixed_outer, not vertical,
-                prefix in ['top', 'left'])
+                prefix in ('top', 'left'))
             generated_boxes.append(box)
 
     # Corner boxes
 
-    for at_keyword, cb_width, cb_height, position_x, position_y in [
+    for at_keyword, cb_width, cb_height, position_x, position_y in (
         ('@top-left-corner', margin_left, margin_top, 0, 0),
         ('@top-right-corner', margin_right, margin_top, page_end_x, 0),
         ('@bottom-left-corner', margin_left, margin_bottom, 0, page_end_y),
         ('@bottom-right-corner', margin_right, margin_bottom,
             page_end_x, page_end_y),
-    ]:
+    ):
         box = make_box(at_keyword, (cb_width, cb_height))
         if not box.is_generated:
             continue
@@ -468,8 +526,8 @@ def page_width_or_height(box, containing_block_size):
      over-constrained, instead of ignoring any margins, the containing block
      is resized to coincide with the margin edges of the page box."
 
-    http://dev.w3.org/csswg/css3-page/#page-box-page-rule
-    http://www.w3.org/TR/CSS21/visudet.html#blockwidth
+    https://drafts.csswg.org/css-page-3/#page-box-page-rule
+    https://www.w3.org/TR/CSS21/visudet.html#blockwidth
 
     """
     remaining = containing_block_size - box.padding_plus_border
@@ -506,9 +564,11 @@ def make_page(context, root_box, page_type, resume_at, page_number,
     and ``resume_at`` indicates where in the document to start the next page,
     or is ``None`` if this was the last page.
 
-    :param page_number: integer, start at 1 for the first page
-    :param resume_at: as returned by ``make_page()`` for the previous page,
-                      or ``None`` for the first page.
+    :param int page_number:
+        Page number, starts at 1 for the first page.
+    :param resume_at:
+        As returned by ``make_page()`` for the previous page, or ``None`` for
+        the first page.
 
     """
     style = context.style_for(page_type)
@@ -543,30 +603,31 @@ def make_page(context, root_box, page_type, resume_at, page_number,
         root_box = root_box.copy_with_children([])
 
     # TODO: handle cases where the root element is something else.
-    # See http://www.w3.org/TR/CSS21/visuren.html#dis-pos-flo
+    # See https://www.w3.org/TR/CSS21/visuren.html#dis-pos-flo
     assert isinstance(root_box, (boxes.BlockBox, boxes.FlexContainerBox))
     context.create_block_formatting_context()
     context.current_page = page_number
-    context.current_page_footnotes = context.reported_footnotes.copy()
+    context.current_page_footnotes = []
     context.current_footnote_area = footnote_area
 
-    if context.reported_footnotes:
-        footnote_area.children = tuple(context.reported_footnotes)
-        context.reported_footnotes = []
-        reported_footnote_area = build.create_anonymous_boxes(
-            footnote_area.deepcopy())
-        reported_footnote_area, _, _, _, _, _ = block_level_layout(
-            context, reported_footnote_area, -inf, None, footnote_area.page,
-            True, [], [], [], False, None)
-        footnote_area.height = reported_footnote_area.height
-        context.page_bottom -= reported_footnote_area.margin_height()
+    reported_footnotes = context.reported_footnotes
+    context.reported_footnotes = []
+    for i, reported_footnote in enumerate(reported_footnotes):
+        context.footnotes.append(reported_footnote)
+        overflow = context.layout_footnote(reported_footnote)
+        if overflow and i != 0:
+            context.report_footnote(reported_footnote)
+            context.reported_footnotes = reported_footnotes[i:]
+            break
 
     page_is_empty = True
     adjoining_margins = []
     positioned_boxes = []  # Mixed absolute and fixed
     out_of_flow_boxes = []
-    broken_out_of_flow = []
-    for box, containing_block, skip_stack in context.broken_out_of_flow:
+    broken_out_of_flow = {}
+    context_out_of_flow = context.broken_out_of_flow.values()
+    context.broken_out_of_flow = broken_out_of_flow
+    for box, containing_block, skip_stack in context_out_of_flow:
         box.position_y = root_box.content_box_y()
         if box.is_floated():
             out_of_flow_box, out_of_flow_resume_at = float_layout(
@@ -579,20 +640,18 @@ def make_page(context, root_box, page_type, resume_at, page_number,
                 skip_stack)
         out_of_flow_boxes.append(out_of_flow_box)
         if out_of_flow_resume_at:
-            broken_out_of_flow.append(
-                (box, containing_block, out_of_flow_resume_at))
-    context.broken_out_of_flow = broken_out_of_flow
+            broken_out_of_flow[out_of_flow_box] = (
+                box, containing_block, out_of_flow_resume_at)
     root_box, resume_at, next_page, _, _, _ = block_level_layout(
         context, root_box, 0, resume_at, initial_containing_block,
-        page_is_empty, positioned_boxes, positioned_boxes, adjoining_margins,
-        False, None)
+        page_is_empty, positioned_boxes, positioned_boxes, adjoining_margins)
     assert root_box
     root_box.children = out_of_flow_boxes + root_box.children
 
     footnote_area = build.create_anonymous_boxes(footnote_area.deepcopy())
-    footnote_area, _, _, _, _, _ = block_level_layout(
+    footnote_area = block_level_layout(
         context, footnote_area, -inf, None, footnote_area.page, True,
-        positioned_boxes, positioned_boxes, [], False, None)
+        positioned_boxes, positioned_boxes)[0]
     footnote_area.translate(dy=-footnote_area.margin_height())
 
     page.fixed_boxes = [
@@ -606,7 +665,6 @@ def make_page(context, root_box, page_type, resume_at, page_number,
     context.finish_block_formatting_context(root_box)
 
     page.children = [root_box, footnote_area]
-    descendants = page.descendants()
 
     # Update page counter values
     _standardize_page_based_counters(style, None)
@@ -630,7 +688,7 @@ def make_page(context, root_box, page_type, resume_at, page_number,
         cached_anchors.extend(x_remake_state.get('anchors', []))
         cached_lookups.extend(x_remake_state.get('content_lookups', []))
 
-    for child in descendants:
+    for child in page.descendants(placeholders=True):
         # Cache target's page counters
         anchor = child.style['anchor']
         if anchor and anchor not in cached_anchors:
@@ -711,6 +769,7 @@ def make_page(context, root_box, page_type, resume_at, page_number,
 
     if page_type.blank:
         resume_at = previous_resume_at
+        next_page = page_maker[page_number - 1][1]
 
     return page, resume_at, next_page
 
@@ -723,7 +782,7 @@ def set_page_type_computed_styles(page_type, html, style_for):
     style_for.set_computed_styles(
         page_type,
         # @page inherits from the root element:
-        # http://lists.w3.org/Archives/Public/www-style/2012Jan/1164.html
+        # https://lists.w3.org/Archives/Public/www-style/2012Jan/1164.html
         root=html.etree_element, parent=html.etree_element,
         base_url=html.base_url)
 
@@ -757,7 +816,6 @@ def remake_page(index, context, root_box, html):
     # Don't modify actual page_maker[index] values!
     # TODO: should we store (and reuse) page_type in the page_maker?
     page_state = copy.deepcopy(initial_page_state)
-    next_page_name = initial_next_page['page']
     first = index == 0
     if initial_next_page['break'] in ('left', 'right'):
         next_page_side = initial_next_page['break']
@@ -771,8 +829,7 @@ def remake_page(index, context, root_box, html):
         (next_page_side == 'left' and right_page) or
         (next_page_side == 'right' and not right_page) or
         (context.reported_footnotes and initial_resume_at is None))
-    if blank:
-        next_page_name = ''
+    next_page_name = '' if blank else initial_next_page['page']
     side = 'right' if right_page else 'left'
     page_type = PageType(side, blank, first, index, name=next_page_name)
     set_page_type_computed_styles(page_type, html, context.style_for)
@@ -787,8 +844,6 @@ def remake_page(index, context, root_box, html):
         context, root_box, page_type, initial_resume_at, page_number,
         page_state)
     assert next_page
-    if blank:
-        next_page['page'] = initial_next_page['page']
     right_page = not right_page
 
     # Check whether we need to append or update the next page_maker item
@@ -835,6 +890,7 @@ def make_all_pages(context, root_box, html, pages):
 
     """
     i = 0
+    reported_footnotes = None
     while True:
         remake_state = context.page_maker[i][-1]
         if (len(pages) == 0 or
@@ -847,16 +903,19 @@ def make_all_pages(context, root_box, html, pages):
             remake_state['anchors'] = []
             remake_state['content_lookups'] = []
             page, resume_at = remake_page(i, context, root_box, html)
+            reported_footnotes = context.reported_footnotes
             yield page
         else:
             PROGRESS_LOGGER.info(
                 'Step 5 - Creating layout - Page %d (up-to-date)', i + 1)
             resume_at = context.page_maker[i + 1][0]
+            reported_footnotes = None
             yield pages[i]
 
         i += 1
-        if resume_at is None and not context.reported_footnotes:
-            # Throw away obsolete pages and broken out-of-flow boxes
+        if resume_at is None and not reported_footnotes:
+            # Throw away obsolete pages and content
             context.page_maker = context.page_maker[:i + 1]
             context.broken_out_of_flow.clear()
+            context.reported_footnotes.clear()
             return
